﻿// Define PI. PI is used for geometry functions cos and sin
var pi = 3.14159265;

// Define the template URL used to display a point in Google maps
var link1 = "http://maps.google.com/maps?f=q&hl=en&geocode=&time=&date=&ttype=&q=";
var link2 = "+";
var link3 = "&ie=UTF8&z=13&iwloc=addr&om=1";

// Validate that the entered observerd Sun altitude is
// an number and is > 0.0.
//
// If the value is not valid the edit fields background
// will be set to red, if valid it will be set to white.
//
// Returns if the field is valid or not.
function validate_Hs() {
  var valid = true;
  var value = Hs.value;

  if (isNaN(value) || value == 0.0) {
    valid = false;
    Hs.background = "#ff0000";
  } else {
    Hs.background = "#ffffff";
  }
  return valid;
}

// Validate that the entered viewer height is a number.
//
// If the value is not valid the edit fields background
// will be set to red, if valid it will be set to white.
//
// Returns if the field is valid or not.
function validate_Dip() {
  var valid = true;
  var value = Dip.value;

  if (isNaN(value)) {
    valid = false;
    Dip.background = "#ff0000";
  } else {
    Dip.background = "#ffffff";
  }
  return valid;
}

// Validate that the entered observation date is valid.
// A date is valid if it can be parsed by JS's date
// class.
//
// If the value is not valid the edit fields background
// will be set to red, if valid it will be set to white.
//
// Returns if the field is valid or not.
function validate_Date() {
  var valid = true;
  var value = oDate.value;

  if (!Date.parse(value)) {
    oDate.background = "#ff0000";
    valid = false;
  } else {
    oDate.background = "#ffffff";
  }

  return valid;
}

// Validates if the given value is a time specification.
//
// Returns if the given value is valid or not.
function validate_Time(t) {
  var it = t;

  if (it.length < 7 || it.length > 8) {
    return false;
  }

  if (it.length < 8) {
    it = "0" + t;
  }

  if (it.charAt(2) != ':' || it.charAt(5) != ':') {
    return false;
  }

  var val = it.substr(0, 2);
  if (isNaN(val) || val < 0 || val > 23) {
    return false;
  }

  var val = it.substr(3, 2);
  if (isNaN(val) || val < 0 || val > 59) {
    return false;
  }

  var val = it.substr(6, 2);
  if (isNaN(val) || val < 0 || val > 59) {
    return false;
  }

  return true;
}

// Validates if the local time is a valid time
// specificaiton.
//
// If the value is not valid the edit fields background
// will be set to red, if valid it will be set to white.
//
// Returns if the field is valid or not.
function validate_Lt() {
  var valid = validate_Time(Lt.value);

  if (!valid) {
    Lt.background = "#ff0000";
  } else {
    Lt.background = "#ffffff";
  }

  return valid;
}

// Validates if the GMT specification is a valid
// time specification.
//
// If the value is not valid the edit fields background
// will be set to red, if valid it will be set to white.
//
// Returns if the field is valid or not.
function validate_Gmt() {
  var valid = validate_Time(Gmt.value);

  if (!valid) {
    Gmt.background = "#ff0000";
  } else {
    Gmt.background = "#ffffff";
  }

  return valid;
}  

// Validates all input fields and enables or disbles the
// "calculate" button and the "Map It" link based on 
// the validity of the fields.
function validate() {
  var valid = true;

  // Validate each field. The calls to the specific validators
  // will set the fields background appropriately. The "valid"
  // variable must be the second clause in the "&&" such that 
  // the first clause will execute even if "valid" == false.
  valid = validate_Hs();
  valid = validate_Date() && valid;
  valid = validate_Dip() && valid;
  valid = validate_Lt() && valid;
  valid = validate_Gmt() && valid;

  // If all input fields are valid enable the "calculate"
  // button and set the text color to black. If a single 
  // field is not valid disable the button and set the 
  // text to a gray.
  calculate.enabled = valid;
  if (!valid) {
    calculate_Label.color = "#909090";
  } else {
    calculate_Label.color = "#000000";
  }
  
  // If the latitude and longitude are valid values then
  // enable the link and set the text to blue. If either are
  // not valid then disable the map link and set the text
  // to gray.
  if (Lat.innerText.length == 0 || Long.innerText.length == 0 || isNaN(Lat.innerText) || isNaN(Long.innerText)) {
    map.color = "#909090";
    map.enabled = false;
  } else {
    map.color = "#0000ff";
    map.enabled = true;
  }
}

function view_onOpen() {
  // Initialize all text strings in the interface from the
  // string catalog.
  Hs_Label.innerText = HS_LABEL;
  Hs.toolTip = HS_TIP;
  Hs.value = HS_DEFAULT;

  oDate_Label.innerText = DATE_LABEL;
  oDate.toolTip = DATE_TIP;
  var today = new Date();
  oDate.value = today.getMonth()+1 + "/" + today.getDate() + "/" + today.getFullYear();

  Dip_Label.innerText = DIP_LABEL;
  Dip.value = DIP_DEFAULT;
  Dip.toolTip = DIP_TIP;

  Lt_Label.innerText = LT_LABEL;
  Lt.value = LT_DEFAULT;
  Lt.toolTip = LT_TIP;

  Gmt_Label.innerText = GMT_LABEL;
  Gmt.value = GMT_DEFAULT;
  Gmt.toolTip = GMT_TIP;

  Lat_Label.innerText = LAT_LABEL;
  Lat.innerText = LAT_DEFAULT;
  Lat.toolTip = LAT_TIP;

  Long_Label.innerText = LONG_LABEL;
  Long.innerText = LONG_DEFAULT;
  Long.toolTip = LONG_TIP;

  calculate.toolTip = CALC_TIP;
  calculate_Label.toolTip = CALC_TIP;
  calculate_Label.innerText = CALC_LABEL;

  map.innerText = MAP_LABEL;
  map.toolTip = MAP_TIP;

  // Validate the input fields.
  validate();
}

// Calculates the latitude and longitude
// based on the input values.
function calculate_onclick() {
  // Radius of the Sun, used for a correction
  // as the sighting is assumed against the
  // lower limb of the Sun.
  var radius = 16/60; 

  // Calculate the dip based on the viewer height
  var dip = feet_to_dip(Dip.value);

  // Calculate the refraction correction based on
  // observed altitude.
  var rc = refraction_correlation(Hs.value);

  // Estimate the Sun's declination based on the observed
  // altitude and the day of the year (Julian date)
  var dec = declination(Hs.value, day_of_year(new Date(Date.parse(oDate.value))));

  // Calculate the corrected altitude correcting for
  // dip, refraction, and the radius of the Sun.
  var ho = Hs.value - dip - rc + radius;

  // Calculate the latitude from the corrected altitude
  // and set it to the output field.
  Lat.innerText = dec - ho + 90;

  // Convert the local time and GMT to their decimal
  // time equivalant so that time math is stright
  // forward.
  var lt = time_to_decimal(Lt.value);
  var gmt = time_to_decimal(Gmt.value);
  
  // Calculate the delta between the two time
  // values and determine if the delta indicates
  // that the longitude is west(-) or east(+) of
  // the prime meridian
  var delta = 0.0;
  if (gmt > lt) {
    delta = -(gmt - lt);
  } else {
    delta = lt - gmt;
  }

  // Calculate the longitude (each hour of the delta
  // is 15 degrees of longitude away from the prime).
  // Set the result to the output field.
  Long.innerText = delta * 15;

  // Set the URL to which the "Map It" link refers to
  // a URL which includes the just calculated latitude
  // and longitude.
  map.href = link1 + Lat.innerText + link2 + Long.innerText + link3;

  // Validate the interface
  validate();
}

// Convert a conventional date to the Julian
// date of day of year.
function day_of_year(day) {
  var onejan = new Date(day.getFullYear(), 0, 1);
  return Math.ceil((day - onejan)/ 86400000) + 1
}

// Convert a conventional time to its
// decimal equivalant.
function time_to_decimal(time) {
  var it = time;
  if (it.length < 8) {
    it = "0" + it;
  }
  
  var sec = it.substr(6, 2);
  var min = it.substr(3, 2);
  var hour = it.substr(0, 2);

  min = min*1 + sec/60.0;

  return hour*1 + min/60.0;
}

// Calculate and return the dip based
// on the height in feet of the viewer.
function feet_to_dip(feet) {
  return 1.144 * Math.sqrt(feet);
}

// Calculate and return the refraction
// correction based on the observed altitude
// of the Sun.
function refraction_correlation(hs) {
  return 60.0/hs;
}

// Calculate and return the Sun's declination
// based on the observed altitude and the day
// of the year of the observation.
function declination(hs, day) {
  return -23.45 * Math.cos(2*pi/365 * (day + 10));
}

